#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Contains dingo tasks to update utc times (computed with SPICE kernels)
into the ROC database."""

from datetime import datetime, timedelta
from pathlib import Path
import uuid

from sqlalchemy import and_, true

from poppy.core.logger import logger
from poppy.core.db.connector import Connector
from poppy.core.task import Task

from roc.dingo.constants import (
    PIPELINE_DATABASE,
    TRYOUTS,
    TIME_WAIT_SEC,
    SQL_LIMIT,
    NAIF_SOLO_ID,
)
from roc.dingo.models.data import SbmLog, BiaSweepLog, LfrKcoeffDump
from roc.dingo.models.packet import TmLog
from roc.dingo.tools import query_db, is_sclk_uptodate, valid_time


class UpdateUtcTime(Task):
    """
    Update UTC times in the
    in ROC database
    """

    plugin_name = "roc.dingo"
    name = "update_utc_time"

    def add_targets(self):
        pass

    @Connector.if_connected(PIPELINE_DATABASE)
    def setup_inputs(self):
        # Get SOLO SPICE kernels (SCLK and LSK)
        self.sclk_file = self.pipeline.get("sclk", default=[None], create=True)[0]
        if not self.sclk_file:
            raise FileNotFoundError(
                f"sclk_file input file is missing!\t[{self.job_id}]"
            )
        self.lsk_file = self.pipeline.get("lsk", default=[None], create=True)[0]
        if not self.sclk_file or not self.lsk_file:
            raise FileNotFoundError(
                f"lsk_file input file is missing!\t[{self.job_id}]"
            )
 
        # Get start_time input value
        self.start_time = valid_time(
            self.pipeline.get("start_time", default=[None])[0],
        )

        # Get end_time input value
        self.end_time = valid_time(
            self.pipeline.get("end_time", default=[None])[0],
        )

        # get a database session
        self.session = Connector.manager[PIPELINE_DATABASE].session

        # Get tryouts from pipeline properties
        self.tryouts = self.pipeline.get("tryouts", default=[TRYOUTS], create=True)[0]

        # Get wait from pipeline properties
        self.wait = self.pipeline.get("wait", default=[TIME_WAIT_SEC], create=True)[0]

        # Retrieve --limit keyword value
        self.limit = self.pipeline.get("limit", default=[SQL_LIMIT], args=True)[0]

        # Database insertion datetime
        self.insert_time = datetime.today()

    def run(self):
        # Define task job ID (long and short)
        self.job_uuid = str(uuid.uuid4())
        self.job_id = self.job_uuid[:8]
        logger.info(f"Task {self.job_id} is starting")
        try:
            self.setup_inputs()
        except Exception:
            logger.exception(f"Initializing inputs has failed for task {self.job_id}!")
            self.pipeline.exit()
            return

        # List of tables for which utc times must be updated
        table_list = [TmLog, SbmLog, BiaSweepLog, LfrKcoeffDump]

        # Loop over tables
        for current_table in table_list:
            # Retrieve table rows which are flagged as predictive in the table
            filters = [current_table.utc_time_is_predictive == true()]
            if self.start_time:
                filters.append(current_table.utc_time >= str(self.start_time))
            if self.end_time:
                filters.append(current_table.utc_time < str(self.end_time))
            filters = and_(*filters)
            rows = query_db(
                self.session,
                current_table,
                filters=filters,
                tryouts=self.tryouts,
                wait=self.wait,
                limit=self.limit,
                raw=True,
            )

            n_rows = len(rows)
            logger.info(
                f"{n_rows} predictive time(s) for {current_table.__tablename__} between {self.start_time} and {self.end_time}\t[{self.job_id}]"
            )
            if n_rows == 0:
                continue

            # Check/Update utc time
            updated_rows = [
                current_row for current_row in rows if self.update_time(current_row)
            ]
            n_rows = len(updated_rows)

            if n_rows > 0:
                # Then commit changes in database
                self.session.commit()
                logger.info(
                    f"{n_rows} updated in {current_table.__tablename__}\t[{self.job_id}]"
                )
            else:
                logger.info(
                    f"No data to update in {current_table.__tablename__}\t[{self.job_id}]"
                )

    def update_time(self, row, naif_id=NAIF_SOLO_ID):
        """
        Check if utc_time in the input row is up-to-date.
        If not, then update it using SPICE.

        :param row: input table row to check/update (dictionary)
        :return:True if row has been updated, False otherwise
        """
        is_uptodate = False
        if is_sclk_uptodate(
            row.utc_time + timedelta(days=1), Path(self.sclk_file).name
        ):
            if hasattr(row, 'cuc_time'):
                cuc_time = row.cuc_time
            else:
                cuc_time = str(row.cuc_coarse_time) + ":" + str(row.cuc_fine_time)

            row.utc_time = self.spice.obt2utc(naif_id, cuc_time)
            row.utc_time_is_predictive = False
            row.insert_time = self.insert_time
            logger.debug(f"{row.id} has been updated [{self.job_id}]")
            is_uptodate = True

        return is_uptodate



# -*- coding: utf-8 -*-
"""empty message

Revision ID: roc_dingo_0006_add_is_removed
Revises: roc_dingo_0005_mv_type_level
Create Date: 2020-11-20 13:53:36.510543

"""

from alembic import op
import sqlalchemy as sa

# revision identifiers, used by Alembic.
revision = "roc_dingo_0006_add_is_removed"
down_revision = "roc_dingo_0005_mv_type_level"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "file_log",
        sa.Column("is_removed", sa.BOOLEAN(), nullable=True),
        schema="pipeline",
    )

    # adding a column with a default value is not possible with Alembic
    # so create the column without the nullable constraint
    # fill the table with the default value (false)
    # after, add the nullable constraint

    op.execute("UPDATE pipeline.file_log SET is_removed = false")
    op.alter_column("file_log", "is_removed", nullable=False, schema="pipeline")

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###

    op.drop_column("file_log", "is_removed", schema="pipeline")
    # ### end Alembic commands ###



# -*- coding: utf-8 -*-
"""Add views to display data grouped by date

Revision ID: roc_dingo_0010_add_view
Revises: roc_dingo_0009_add_bia
Create Date: 2021-06-14 11:29:28.513231

"""

from alembic import op

# revision identifiers, used by Alembic.
revision = "roc_dingo_0010_add_view"
down_revision = "roc_dingo_0009_add_bia"
branch_labels = None
depends_on = None


def upgrade():
    op.execute("COMMIT")
    op.execute("""CREATE OR REPLACE VIEW pipeline.file_log_per_date AS
        select date(start_time) as date,
            count(state='OK' or null) as OK,
            count(state='ERROR' or null) as ERROR,
            count(state='WARNING' or null) as WARNING,
            count(*)
        from pipeline.file_log f1
        where f1.start_time is not null
        and f1.is_latest and not f1.is_removed
        group by (date(start_time))
        order by date
    """)

    op.execute("""CREATE OR REPLACE VIEW pipeline.file_log_per_date_wo_sk AS
        select date(start_time) as date,
            count(state='OK' or null) as OK,
            count(state='ERROR' or null) as ERROR,
            count(state='WARNING' or null) as WARNING,
            count(*)
        from pipeline.file_log f1
        where f1.start_time is not null
        and level != 'SK'
        and f1.is_latest and not f1.is_removed
        group by (date(start_time))
        order by date
    """)


def downgrade():
    op.execute("COMMIT")
    op.execute("DROP VIEW IF EXISTS pipeline.file_log_per_date")
    op.execute("DROP VIEW IF EXISTS pipeline.file_log_per_date_wo_sk")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Tests module for the roc.dingo update_sbm_retrieved command and related tasks.
"""

import glob
import os
import shlex
import tarfile

import unittest.mock as mock
from pathlib import Path

import pytest

from poppy.core.conf import Settings
from poppy.core.logger import logger
from poppy.core.generic.requests import download_file
from poppy.core.test import CommandTestCase

from roc.dingo.models.packet import TmLog
from roc.dingo.models.data import SbmLog

# Tests on roc.dingo update_sbm_retrieved command
from roc.dingo.tests.test_dingo import DingoTest
from roc.dingo.constants import DINGO_CACHE_DIR

EXPECTED_ROWS = (
    {
        "info": {
            "TM_TDS_SCIENCE_SBM1_RSWF": {"size": 2916, "count": 27, "percent": 1.25},
            "TM_LFR_SCIENCE_SBM1_BP1_F0": {
                "size": 777330,
                "count": 2879,
                "percent": 100.0,
            },
            "TM_LFR_SCIENCE_SBM1_BP2_F0": {
                "size": 495360,
                "count": 720,
                "percent": 100.0,
            },
            "TM_LFR_SCIENCE_SBM1_CWF_F1": {"size": 0, "count": 0, "percent": 0.0},
            "TM_TDS_SCIENCE_SBM1_RSWF_C": {
                "size": 2221930,
                "count": 2132,
                "percent": 98.70370370370371,
            },
            "TM_LFR_SCIENCE_SBM1_CWF_F1_C": {
                "size": 13138060,
                "count": 8776,
                "percent": 99.97721576668945,
            },
        },
        "size": 16635596,
        "percent": 99.9827298675983,
    },
    {
        "info": {
            "TM_TDS_SCIENCE_SBM1_RSWF": {
                "size": 4644,
                "count": 43,
                "percent": 1.990740740740741,
            },
            "TM_LFR_SCIENCE_SBM1_BP1_F0": {
                "size": 777330,
                "count": 2879,
                "percent": 100.0,
            },
            "TM_LFR_SCIENCE_SBM1_BP2_F0": {
                "size": 495360,
                "count": 720,
                "percent": 100.0,
            },
            "TM_LFR_SCIENCE_SBM1_CWF_F1": {"size": 0, "count": 0, "percent": 0.0},
            "TM_TDS_SCIENCE_SBM1_RSWF_C": {
                "size": 2266296,
                "count": 2117,
                "percent": 98.00925925925927,
            },
            "TM_LFR_SCIENCE_SBM1_CWF_F1_C": {
                "size": 13125590,
                "count": 8775,
                "percent": 99.96582365003418,
            },
        },
        "size": 16669220,
        "percent": 99.99145591250854,
    },
    {
        "info": {
            "TM_TDS_SCIENCE_SBM1_RSWF": {
                "size": 3672,
                "count": 34,
                "percent": 1.574074074074074,
            },
            "TM_LFR_SCIENCE_SBM1_BP1_F0": {
                "size": 777330,
                "count": 2879,
                "percent": 100.0,
            },
            "TM_LFR_SCIENCE_SBM1_BP2_F0": {
                "size": 495360,
                "count": 720,
                "percent": 100.0,
            },
            "TM_LFR_SCIENCE_SBM1_CWF_F1": {"size": 0, "count": 0, "percent": 0.0},
            "TM_TDS_SCIENCE_SBM1_RSWF_C": {
                "size": 2194002,
                "count": 2126,
                "percent": 98.42592592592592,
            },
            "TM_LFR_SCIENCE_SBM1_CWF_F1_C": {
                "size": 13115610,
                "count": 8775,
                "percent": 99.96582365003418,
            },
        },
        "size": 16585974,
        "percent": 99.99145591250854,
    },
    {
        "info": {
            "TM_TDS_SCIENCE_SBM1_RSWF": {
                "size": 4212,
                "count": 39,
                "percent": 1.8055555555555554,
            },
            "TM_LFR_SCIENCE_SBM1_BP1_F0": {
                "size": 777330,
                "count": 2879,
                "percent": 100.0,
            },
            "TM_LFR_SCIENCE_SBM1_BP2_F0": {
                "size": 495360,
                "count": 720,
                "percent": 100.0,
            },
            "TM_LFR_SCIENCE_SBM1_CWF_F1": {"size": 0, "count": 0, "percent": 0.0},
            "TM_TDS_SCIENCE_SBM1_RSWF_C": {
                "size": 2259828,
                "count": 2121,
                "percent": 98.19444444444444,
            },
            "TM_LFR_SCIENCE_SBM1_CWF_F1_C": {
                "size": 13196088,
                "count": 8775,
                "percent": 99.96582365003418,
            },
        },
        "size": 16732818,
        "percent": 99.99145591250854,
    },
    {
        "info": {
            "TM_TDS_SCIENCE_SBM1_RSWF": {
                "size": 1512,
                "count": 14,
                "percent": 0.6481481481481481,
            },
            "TM_LFR_SCIENCE_SBM1_BP1_F0": {
                "size": 777330,
                "count": 2879,
                "percent": 100.0,
            },
            "TM_LFR_SCIENCE_SBM1_BP2_F0": {
                "size": 495360,
                "count": 720,
                "percent": 100.0,
            },
            "TM_LFR_SCIENCE_SBM1_CWF_F1": {"size": 0, "count": 0, "percent": 0.0},
            "TM_TDS_SCIENCE_SBM1_RSWF_C": {
                "size": 2124022,
                "count": 2146,
                "percent": 99.35185185185185,
            },
            "TM_LFR_SCIENCE_SBM1_CWF_F1_C": {
                "size": 13042384,
                "count": 8775,
                "percent": 99.96582365003418,
            },
        },
        "size": 16440608,
        "percent": 99.99145591250854,
    },
)


class TestDingoUpdateSbmRetrievedCommand(CommandTestCase):
    def setup_method(self, method):
        super().setup_method(method)

        # empty the database
        self.session.query(TmLog).delete()
        self.session.query(SbmLog).delete()
        self.session.flush()

    def teardown_method(self, method):
        super().teardown_method(method)

    @pytest.mark.parametrize(
        "idb_source,idb_version, exclude_tm_apid",
        [
            ("MIB", "20200131", "1212 1228 1244 1260 1204 1220 1236 1252 1269 1300"),
        ],
    )
    def test_update_sbm_retrieved(self, idb_source, idb_version, exclude_tm_apid):
        # Command name
        cmd = "update_sbm_retrieved"

        # First get test data to insert (i.e. L0 input file(s))
        test_data_tarball = "test_data.tar.gz"

        self.install_dir = os.environ.get("ROC_TEST_DATA_PATH", DINGO_CACHE_DIR)

        test_data_dir = os.path.join(self.install_dir, cmd)
        Path(test_data_dir).mkdir(exist_ok=True, parents=True)
        test_data_path = os.path.join(test_data_dir, test_data_tarball)
        if not os.path.isfile(test_data_path):
            # Build test data tarball URL
            test_data_url = "/".join([DingoTest.base_url, cmd, test_data_tarball])
            # Get username/password
            test_dingo = DingoTest()

            download_file(
                test_data_path,
                test_data_url,
                auth=(test_dingo.username, test_dingo.password),
            )

        # Extract tarball
        if tarfile.is_tarfile(test_data_path):
            logger.info(f"Extracting {test_data_path} ...")
            with tarfile.open(test_data_path, "r:*") as tarball:
                tarball.extractall(path=test_data_dir, filter="fully_trusted")
        else:
            raise tarfile.ReadError(f"{test_data_path} is not a valid tarball!")

        # Get list of input files
        input_dir = os.path.join(test_data_dir, "inputs")
        input_file = list(glob.glob(os.path.join(input_dir, "solo_L0_rpw*.h5")))
        if not input_file:
            raise FileNotFoundError(f"No input L0 file found in {input_dir}")
        elif len(input_file) > 1:
            raise ValueError("/inputs must contain only one L0 test file!")
        else:
            # Make sure to have only file
            input_file = input_file[0]

        # Check that SPICE kernels are present in ./data/spice_kernels folder
        spice_kernels_dir = DingoTest.get_spice_kernel_dir()
        if not os.path.isdir(spice_kernels_dir):
            raise FileNotFoundError(
                f"No SPICE kernel set found in {spice_kernels_dir}!"
            )
        else:
            # Get latest version of SCLK et LSK SPICE kernels
            sclk = max(
                (Path(spice_kernels_dir) / "sclk").glob("solo_ANC_soc-sclk_*.tsc"),
                key=os.path.getctime,
            )
            lsk = max(
                (Path(spice_kernels_dir) / "lsk").glob("naif*.tls"),
                key=os.path.getctime,
            )

        # First insert data into database
        command_to_run = " ".join(
            [
                "pop",
                "dingo",
                "--idb-source",
                idb_source,
                "--idb-version",
                idb_version,
                "--sclk",
                str(sclk),
                "--lsk",
                str(lsk),
                "l0_to_db",
                "-l0",
                input_file,
                "--exclude-tm-apid",
                exclude_tm_apid,
                "-ll",
                "INFO",
            ]
        )

        # define the required plugins
        plugin_list = ["poppy.pop", "roc.dingo"]

        # run the command
        # force the value of the plugin list
        with mock.patch.object(
            Settings,
            "configure",
            autospec=True,
            side_effect=self.mock_configure_settings(
                dictionary={"PLUGINS": plugin_list}
            ),
        ):
            self.run_command(shlex.split(command_to_run))

        # Command to test
        command_to_test = " ".join(["pop", "dingo", cmd, "-ll", "INFO"])
        # run the command
        # force the value of the plugin list
        with mock.patch.object(
            Settings,
            "configure",
            autospec=True,
            side_effect=self.mock_configure_settings(
                dictionary={"PLUGINS": plugin_list}
            ),
        ):
            self.run_command(shlex.split(command_to_test))

        # Check that test data are in the database
        logger.debug("Querying sbm_log table in database ...")
        rows = self.session.query(SbmLog).order_by(SbmLog.utc_time).all()

        for i, row in enumerate(rows):
            current_retrieved = row.retrieved
            current_expected = EXPECTED_ROWS[i]

            assert all(
                (current_expected.get(k) == v for k, v in current_retrieved.items())
            )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Tests module for the roc.dingo plugin.
"""

import pytest
import os.path as osp
from argparse import Namespace

from poppy.core.pipeline import Pipeline
from poppy.core.logger import logger
from poppy.core.test import TaskTestCase

from roc.dingo.tasks import StoreDelivered
from roc.dingo.models.file import FileLog
from roc.dingo.tests.test_log_file import DingoLogFileTest


# Tests on roc.dingo.tasks.log_file methods
class TestDingoLogFileStoreDelivered(TaskTestCase):
    @pytest.mark.parametrize(
        "root, delivered_root, data_file, public_data_file, "
        "nb_before, nb_pub_before, nb_stored, nb_removed",
        [
            (
                "logfile_to_rocdb/DATA-1/",
                "logfile_to_rocdb/DELIVERED-1/",
                "roc_data_files_all.dat",
                "roc_data_files_public_filename.dat",
                61,
                31,
                31,
                0,
            )
        ],
    )
    def test_log_file_store_delivered(
        self,
        root,
        delivered_root,
        data_file,
        public_data_file,
        nb_before,
        nb_pub_before,
        nb_stored,
        nb_removed,
    ):
        """
        Start from an empty database
        populates it by hand with files from data_file
        Launch task and check result

        :return:
        """

        logger.info("test_store_public_filename() ")

        # --- set up arguments ---
        data_test_path = DingoLogFileTest.get_test_data_path()
        root = osp.join(data_test_path, root)
        delivered_root = osp.join(data_test_path, delivered_root)
        args = Namespace(delivered=delivered_root, dry_run=False)

        # --- database setup ---
        session = DingoLogFileTest.setup_session()

        # --- define data test --
        data1 = DingoLogFileTest.open_test_data(data_file, root)

        # files with their public_filename
        public_filename = DingoLogFileTest.read_data(public_data_file)
        public_filename = [line.split() for line in public_filename]
        public_filename = {line[0]: line[1] for line in public_filename}

        # populate database
        for item, data in data1.items():
            file_log = DingoLogFileTest.data_to_file_log(data, root)
            if file_log.basename in public_filename:
                file_log.public_filename = public_filename[file_log.basename]
            session.add(file_log)
        session.flush()

        # --- get database entries ---
        db_before = session.query(FileLog).all()
        assert len(db_before) == nb_before

        db_before_pub = (
            session.query(FileLog).filter(FileLog.public_filename.isnot(None)).all()
        )
        assert len(db_before_pub) == nb_pub_before

        # --- initialise task & pipeline --
        self.task = StoreDelivered()
        self.task.pipeline = Pipeline(args)
        self.task.instantiate_targets()
        self.task.session = session

        # --- initialise inputs ---

        # --- run the task ---
        self.run_task()

        # --- get database entries ---
        db_after = session.query(FileLog).filter(FileLog.is_delivered == True).all()  # noqa : E712

        # --- get results ---
        store_results = self.task.outputs["store_delivered_results"].data

        # --- make assertions ---

        assert len(db_after) == nb_stored
        assert store_results["files_stored"] == nb_stored
        assert store_results["removed_files"] == nb_removed

        # --- close session
        session.close()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import glob
import os
import argparse
import time
from datetime import datetime
import hashlib
import shutil

import h5py
import math
import numpy
from typing import Union, Type

import pandas as pd
import sqlalchemy
from sqlalchemy.exc import IntegrityError, NoResultFound, MultipleResultsFound
from sqlalchemy.dialects.postgresql import insert
from sqlalchemy.sql.expression import true
from sqlalchemy import and_

from poppy.core.logger import logger
from poppy.core.db.handlers import get_or_create_with_info

from spice_manager import SpiceManager

from roc.rpl.packet_parser import raw_to_eng
from roc.idb.models.idb import IdbRelease

from roc.dingo.constants import (
    TIME_DAILY_STRFORMAT,
    TIME_INPUT_STRFORMAT,
    SQL_LIMIT,
    TRYOUTS,
    TIME_WAIT_SEC,
    TF_CP_BIA_P011_SRDB_ID,
    IDB_SOURCE,
    NAIF_SOLO_ID,
)

from roc.dingo.exceptions import DingoException, DbQueryError

__all__ = [
    "cuc2utc",
    "hex_to_bytes",
    "insert_on_conflict_nothing",
    "insert_on_conflict_update",
    "valid_time",
    "valid_date",
    "valid_dir",
    "valid_data_version",
    "glob_paths",
    "get_packet_sha",
    "get_dict_sha",
    "get_current_idb",
    "compute_apid",
    "compute_pkt_seq_control",
    "load_spice",
    "insert_in_db",
    "safe_move",
    "get_or_create_in_db",
    "query_db",
    "delete_in_db",
    "actual_sql",
    "gen_sql_filters",
    "is_sclk_uptodate",
    "get_columns",
    "bulk_insert",
    "sbm_qf_eng",
    "raw_to_na",
    "compute_hfr_list_freq",
    "round_up",
]


def cuc2utc(
    spice_object: SpiceManager,
    cuc_time: Union[list, tuple, numpy.ndarray],
    naif_id: int = NAIF_SOLO_ID,
) -> str:
    """
    Convert input RPW CUC time into UTC time

    :param spice_object: SpiceManager class instance
    :param cuc_time: CUC time
    :param naif_id: NAIF SPICE ID
    :return: UTC time as returned by SpiceManager.obt2utc() method
    """
    obt_time = spice_object.cuc2obt(cuc_time)
    return spice_object.obt2utc(naif_id, obt_time)


def hex_to_bytes(string: str) -> bytearray:
    """
    Convert a hexadecimal string into a byte array.

    :param string: hexadecimal string to convert
    :return: input string as a byte array
    """
    # transform to an array of bytes
    return bytearray.fromhex(string)


def insert_on_conflict_nothing(
    table: Type[object],
    session: sqlalchemy.orm.Session,
    data_to_insert: list,
    commit: bool = True,
    index_elements: list = None,
    constraint: str = None,
) -> int:
    """
    Apply Insert.on_conflict_do_nothing() method in case of index conflict(s).

    :param table: name of the database table
    :type table: See sqlalchemy.dialects.postgresql.insert
    :param session: database session object
    :type session: sqlalchemy.orm.Session
    :param data_to_insert: data to be inserted as a list of dictionaries
    :type data_to_insert: list
    :param commit: whether to run session.commit() or not (default True)
    :type commit: bool
    :param index_elements: A sequence consisting of string column names,
    Column objects, or other column expression objects that will be used to infer a target index.
    :type index_elements: list
    :param constraint: The name of a unique or exclusion constraint on the table,
    or the constraint object itself if it has a .name attribute.
    :type constraint: str
    :return: number of data inserted
    """

    stmt = (
        insert(table)
        .values(data_to_insert)
        .on_conflict_do_nothing(index_elements=index_elements, constraint=constraint)
    )
    result = session.execute(stmt)
    if commit:
        session.commit()

    return result.rowcount


def insert_on_conflict_update(
    table: Type[object],
    session: sqlalchemy.orm.Session,
    data_to_insert: list,
    index_elements: list = None,
    constraint: str = None,
    set_: dict = None,
    set_from_excluded: bool = False,
    commit: bool = True,
) -> int:
    """
    Apply Insert.on_conflict_do_update() method in case of index conflict(s).
    See sqlalchemy.dialects.postgresql.Insert.on_conflict_do_update() documentation for details.

    :param table: name of the database table
    :type table: See sqlalchemy.dialects.postgresql.insert
    :param session: database session object
    :type session: sqlalchemy.orm.Session
    :param data_to_insert: data to be inserted as a list of dictionaries
    :type data_to_insert: list
    :param index_elements: A sequence consisting of string column names,
    Column objects, or other column expression objects that will be used to infer a target index.
    :type index_elements: list
    :param constraint: The name of a unique or exclusion constraint on the table,
    or the constraint object itself if it has a .name attribute.
    :type constraint: str
    :param set_: A dictionary or other mapping object where the keys are either
    names of columns in the target table, or Column objects or
    other ORM-mapped columns matching that of the target table,
    and expressions or literals as values, specifying the SET actions to take.
    :type set_: dict
    :param set_from_excluded: If True then fill set_ input automatically from stmt.excluded columns
    :type set_from_excluded: bool
    :param commit: whether to run session.commit() or not (default True)
    :type commit: bool
    :return: number of data inserted
    """

    stmt = insert(table).values(data_to_insert)

    if set_from_excluded:
        set_ = {col: getattr(stmt.excluded, col) for col in data_to_insert[0]}

    stmt = stmt.on_conflict_do_update(
        index_elements=index_elements, constraint=constraint, set_=set_
    )

    result = session.execute(stmt)
    if commit:
        session.commit()

    return result.rowcount


def valid_dir(str_dir: Union[str, list]) -> str:
    """
    Make sure to have a valid input directory.

    :param str_dir: 1-element list or string containing the path to the directory
    :type str_dir: Union[str, list]
    :return: input directory if valid
    :rtype: str
    """
    try:
        if isinstance(str_dir, list):
            str_dir = str_dir[0]
        if os.path.isdir(str_dir):
            return str_dir
        else:
            raise IsADirectoryError
    except IsADirectoryError:
        raise_error(
            f"Input directory not found! ({str_dir})", exception=IsADirectoryError
        )
    except ValueError:
        raise_error(f"Input directory is not valid! ({str_dir})", exception=ValueError)
    except Exception as e:
        raise_error(f"Problem with input directory! ({str_dir})", exception=e)


def glob_paths(paths: Union[str, list]) -> list:
    """
    Make sure input paths are expanded
    (can be used to avoid any path with wildcard pattern)

    :param paths: list of input paths to glob
    :type paths: Union[str, list]
    :return: list of paths after glob filtering
    :rtype: list
    """
    globbed_paths = []
    if not isinstance(paths, list):
        paths = [paths]

    for current_path in paths:
        globbed_paths.extend(glob.glob(current_path))

    return globbed_paths


def round_up(n: float, decimals: int = 0) -> float:
    """
    Compute round of input float

    :param n: input float
    :type n: float
    :param decimals: round precision
    :type decimals: int
    :return: rounded float
    :rtype: float
    """
    multiplier = 10**decimals
    return math.ceil(n * multiplier) / multiplier


def raise_error(message: str, exception: Type[Exception] = DingoException):
    """Add an error entry to the logger and raise an exception."""
    logger.error(message)
    raise exception(message)


def valid_time(t: str, str_format: str = TIME_INPUT_STRFORMAT) -> datetime:
    """
    Validate input datetime string format.

    :param t: input datetime string
    :type t: str
    :param str_format: expected datetime string format
    :type str_format: str
    :return: datetime object with input datetime info
    :rtype: datetime.datetime
    """
    if t and isinstance(t, str):
        try:
            t = datetime.strptime(t, str_format)
        except ValueError:
            logger.error(f"Not a valid time: '{t}'!")
            raise

    return t


def valid_date(
    t_in: str, str_tformat: str = TIME_DAILY_STRFORMAT
) -> Union[datetime, None]:
    """
    Validate input date string format.

    :param t_in: input date string
    :type t_in: str
    :param str_tformat: expected date string format
    :type str_tformat: str
    :return: date object with input date info
    :rtype: datetime.datetime
    """
    t_out = t_in
    if t_in and isinstance(t_in, str):
        try:
            t_out = datetime.strptime(t_in, str_tformat).date()
        except ValueError:
            argparse.ArgumentTypeError(f"Not a valid date: '{t_in}'!")
    return t_out


def valid_data_version(data_version: Union[str, int]) -> str:
    """
    Make sure to have a valid data version.

    :param data_version: integer or string containing the data version
    :type data_version: Union[str, int]
    :return: string containing valid data version (i.e., 2 digits string):
    :rtype: str
    """
    try:
        data_version = int(data_version)
        return f"{data_version:02d}"
    except ValueError:
        raise_error(
            f"Input value for --data-version is not valid! \
                     ({data_version})"
        )


def get_packet_sha(packet_data: Union[h5py.Group, dict]) -> str:
    """
    Compute the SHA256 of the input packet.
    TM sha is computed from binary
    TC sha is computed from packet name (SRDB ID), execution UTC time and status

    :param packet_data: data of the input packet
    :type packet_data: h5py.Group or dict
    :return: string containing SHA (hexdigest)
    :rtype: str
    """
    sha = None
    packet_name = packet_data["palisade_id"]
    if packet_name.startswith("TC"):
        raw_sha = hashlib.sha256()
        raw_sha.update(packet_data["srdb_id"].encode("utf-8"))
        raw_sha.update(packet_data["utc_time"].isoformat().encode("utf-8"))
        raw_sha.update(packet_data["tc_exe_state"].encode("utf-8"))
        sha = str(raw_sha.hexdigest())
    elif packet_name.startswith("TM"):
        raw_sha = hashlib.sha256()
        raw_sha.update(packet_data["binary"].encode("utf-8"))
        sha = str(raw_sha.hexdigest())
    else:
        logger.error(f"Unknown packet name: {packet_name}")

    return sha


def get_dict_sha(dict_to_hash: dict, include=None, exclude=None) -> str:
    """
    Compute a SHA256 from the values in an input dictionary

    :param dict_to_has: input dictionary to hash
    :type dict_to_hash: dict
    :param include: keywords to include.
    :type include: list
    :param exclude: keywords to ignore
    :type exclude: list
    :return: SHA as a hexa digest string
    """
    sha = hashlib.sha256()
    for key, val in dict_to_hash.items():
        if include and key not in include:
            continue
        if exclude and key in exclude:
            continue
        if isinstance(val, datetime):
            sha.update(val.isoformat().encode("utf-8"))
        else:
            sha.update(str(val).encode("utf-8"))

    return str(sha.hexdigest())


def compute_apid(
    process_id: Union[int, numpy.uint8], packet_category: Union[int, numpy.uint8]
) -> int:
    """
    Compute the APID using the process_id and the packet_category
    APID = |0000000|0000|
        process_id | packet_category

    :param process_id: process id
    :type process_id: Union[int, numpy.uint8]
    :param packet_category: packet category
    :type packet_category: Union[int, numpy.uint8]
    :return: APID
    :rtype: int
    """
    return int((process_id << 4) + packet_category)  # 4 bits shift


def compute_pkt_seq_control(segmentation_grouping_flag, sequence_cnt):
    """
    Compute Packet Sequence Control field for a given packet

    :param segmentation_grouping_flag: Integer storing the packet segmentation_grouping_flag
    :param sequence_cnt: Integer containing the packet sequence counter
    :return: Packet Sequence Control (16-bits)
    """
    return (segmentation_grouping_flag << 14) + sequence_cnt


def load_spice(spice_kernels=[]):
    """
    Load SpiceManager instance with input SOLO kernels

    :param spice_kernels: List of input kernels to load in SPICE
    :return: SpiceManager instance
    """
    from spice_manager import SpiceManager

    return SpiceManager(spice_kernels, logger=logger)


def insert_in_db(
    session,
    model,
    data_to_insert,
    update_fields={},
    update_fields_kwargs={},
    tryouts=TRYOUTS,
    wait=TIME_WAIT_SEC,
):
    """
    Insert a data entry in the database

    :param session: open database session
    :param model: database model to use for input data
    :param data_to_insert: data to insert as an entry in the database
    :param update_fields: If entry already exists in the database,
                            then update only fields provided in this dictionary
    :param update_fields_kwargs: dictionary to pass to filter_by() method when query for updating
    :param tryouts: number of tries
    :param wait: seconds to wait between two tries
    :return: insertion status (0=OK, -1=NOK, 1=Already inserted, 2=Updated)
    """
    # if update_fields_kwargs not provided,
    # then use data_to_insert dictionary
    if not update_fields_kwargs:
        update_fields_kwargs = data_to_insert

    for i in range(tryouts):
        try:
            # Add current data as a new entry of the model table
            # in the database
            session.add(model(**data_to_insert))
            # Commit database change(s)
            session.commit()
        except IntegrityError:
            session.rollback()
            logger.debug(f"{data_to_insert} already inserted")
            # If entry already exists,
            # check if fields needs to be updated
            if update_fields:
                logger.debug(f"Updating {update_fields} in the database ...")
                instance = session.query(model).filter_by(**update_fields_kwargs).one()
                # update it
                for field in update_fields or {}:
                    if getattr(instance, field) != update_fields[field]:
                        setattr(instance, field, update_fields[field])
                session.commit()
                insert_status = 2
            else:
                insert_status = 1
            break
        except Exception as e:
            session.rollback()
            logger.error(f"Inserting {data_to_insert} has failed!")
            logger.debug(e)
            insert_status = -1
            time.sleep(wait)
        else:
            logger.debug(f"{data_to_insert} inserted")
            insert_status = 0
            break

    return insert_status


def safe_move(src, dst, ignore_patterns=[], copy=False):
    """
    Perform a safe move of a file or directory.

    :param src: string containing the path of the file/directory to move
    :param dst: string containing the path of the target file/directory
    :param ignore_patterns: string containing the file patterns to ignore (for copytree only)
    :param copy: if passed, copy files/dirs
    :return: True if the move has succeeded, False otherwise
    """

    # Initialize output
    is_copied = False

    # First do a copy...
    try:
        if os.path.isfile(src):
            shutil.copy(src, dst, follow_symlinks=True)
        elif os.path.isdir(src):
            shutil.copytree(
                src,
                dst,
                ignore=shutil.ignore_patterns(ignore_patterns),
                dirs_exist_ok=True,
            )
    except Exception as e:
        logger.exception(f"Cannot move {src} into {dst}!")
        raise e
    else:
        # then delete if the file has well copied
        if os.path.exists(dst):
            is_copied = True
            if os.path.isfile(src) and not copy:
                os.remove(src)
            elif os.path.isdir(src) and not copy:
                shutil.rmtree(src)

    return is_copied


def query_db(
    session,
    model,
    filters=None,
    limit=SQL_LIMIT,
    order_by=None,
    to_dict=None,
    is_one=False,
    tryouts=TRYOUTS,
    wait=TIME_WAIT_SEC,
    raise_exception=False,
    raw=False,
):
    """
    Query entries from the ROC pipeline database.

    :param session: ROC database open session
    :param model: Table model class
    :param filters: query filters passed as a SQL expression object
    :param limit: Integer containing max. number of returned rows
    :param order_by: Sort returned rows by column value passed in order_by keyword
    :param to_dict: If passed with argument, apply the pandas.DataFrame.to_dict(orient=arg) method
    :param is_one: If True, only one entry is expected.
    :param tryouts: Number of query retries
    :param wait: number of seconds to wait between two retries
    :param raise_exception: If True, raise an exception
    :param raw: If True, return entries as returned by SQLAlchemy.query.all() method
    :return: entries found in the database as returned by pandas.read_sql() method
    """
    # Initialize output
    table_entries = None
    # Run query
    has_failed = True
    for current_try in range(tryouts):
        try:
            if isinstance(model, list) or isinstance(model, tuple):
                query = session.query(*model)
            else:
                query = session.query(model)
            if filters is not None:
                query = query.filter(filters)
            if order_by is not None:
                query = query.order_by(order_by)
            query.limit(int(limit))
            logger.debug(f"Querying database: {actual_sql(query)} ...")
            if raw:
                table_entries = query.all()
                nrec = len(table_entries)
            else:
                table_entries = pd.read_sql(query.statement, session.bind)
                nrec = table_entries.shape[0]
            if nrec == 0:
                raise NoResultFound
            elif is_one and nrec != 1:
                raise MultipleResultsFound
        except MultipleResultsFound:
            logger.exception("Query has returned multiple results!")
            break
        except NoResultFound:
            logger.debug("No results found")
            has_failed = False
            break
        except Exception as e:
            logger.error("Query has failed!")
            logger.debug(e)
        else:
            # logger.debug(f'{query.count()} entries found in the {model} table for {filters}')
            # Convert returned entries into list of lists
            if not raw and to_dict:
                table_entries = table_entries.to_dict(orient=to_dict)

            has_failed = False
            break
        logger.debug(f"Retrying query ({current_try} on {tryouts})")
        time.sleep(wait)

    if has_failed and raise_exception:
        raise DbQueryError(f"Querying database with model {model} has failed!")

    return table_entries


def get_or_create_in_db(
    session, model, entry, kwargs=None, tryouts=TRYOUTS, wait=TIME_WAIT_SEC
):
    """
    Insert input entry to pipeline.data_queue table

    :param session: database session object
    :param model: Database table model class
    :param entry: A dictionary containing column:value to insert in the table
    :param kwargs: A dictionary containing the column:value to use to get data
    :param tryouts: number of tries to insert data
    :param wait: seconds to wait between two tries
    :return: (table entry created, database request status flag, creation status flag)
    """
    job = None
    created = None
    done = False
    if not kwargs:
        kwargs = entry
    for current_try in range(tryouts):
        try:
            job, created = get_or_create_with_info(
                session, model, **kwargs, create_method_kwargs=entry
            )
        except Exception as e:
            logger.error(
                f"Cannot query {model.__tablename__} [retry {tryouts - current_try}]"
            )
            logger.debug(e)
            time.sleep(wait)
        else:
            done = True
            break

    return job, done, created


def bulk_insert(
    session,
    model,
    data_to_insert,
    tryouts=TRYOUTS,
    wait=TIME_WAIT_SEC,
    exists_ok=False,
    raise_exception=True,
):
    """
    Run the bulk_insert_mappings() SQLAlchemy method
    to insert a bulk of data into the database.

    :param session: current database session
    :param model: database table model class
    :param data_to_insert: List of dictionaries to insert in the database
    :param tryouts: number of insertion attempts
    :param wait: seconds to wait between two attempts
    :param raise_exception: if true raise an exception
    :param exists_ok: If True then insertion is OK if entry is already found in the database
    :return: True if insertion has worked, False otherwise
    """
    has_worked = False
    raised_exc = Exception()
    for current_try in range(tryouts):
        try:
            session.bulk_insert_mappings(model, data_to_insert)
            session.commit()
        except IntegrityError as e:
            session.rollback()
            raised_exc = e
            if exists_ok:
                has_worked = True
            break
        except Exception as e:
            session.rollback()
            time.sleep(wait)
            raised_exc = e
        else:
            has_worked = True
            break

    if not has_worked and raise_exception:
        raise raised_exc

    return has_worked


def delete_in_db(session, model, filters=None, tryouts=TRYOUTS, wait=TIME_WAIT_SEC):
    """
    Delete row(s) of a table in the database

    :param session: database session
    :param model: Table model
    :param filters: list of filters
    :param tryouts: number of tries
    :param wait: seconds to wait between two tries
    :return: True if deletion has succeeded, False otherwise
    """
    is_deleted = False
    for current_try in range(tryouts):
        try:
            query = session.query(model)
            if filters:
                query.filter(filters)
            query.delete()
            session.commit()
        except NoResultFound:
            is_deleted = True
            break
        except Exception as e:
            logger.error(f"Deleting {model} with filters {filters} has failed!")
            logger.debug(e)
            time.sleep(wait)
        else:
            is_deleted = True
            break

    return is_deleted


def actual_sql(sqlalchemy_query):
    """
    convert input Sqlalchemy query into explicit SQL syntax query

    :param sqlalchemy_query: input Sqlalchemy query object
    :return: string with corresponding SQL syntax
    """
    return str(
        sqlalchemy_query.statement.compile(compile_kwargs={"literal_binds": True})
    )


def is_sclk_uptodate(current_datetime, sclk_basename):
    """
    Check if the input SOLO SCLK SPICE kernel is newer or older than a give datetime

    :param current_datetime: datetime.datetime object to compare with SOLO SCLK SPICE kernel date
    :param sclk_basename: string containing SOLO SCLK SPICE kernel basename
    :return: True if SCLK SPICE kernel is newer than current_datetime
            , False otherwise
    """
    # Get the date of the SCLK SPICE kernel
    sclk_date = datetime.strptime(
        sclk_basename.split("_")[3],
        TIME_DAILY_STRFORMAT,
    ).date()

    return sclk_date > current_datetime.date()


def get_columns(model, remove=[]):
    """
    Get list of table columns for input model class

    :param model: Table model class
    :param remove: list of columns to remove
    :return: list of table columns
    """
    # Get columns
    columns = model.__table__.columns.keys()
    for key in remove:
        columns.remove(key)
    return columns


def raw_to_na(raw_values, idb_source=IDB_SOURCE, idb_version=None):
    """
    Convert input raw values of bias current into physical units (nA)

    :param raw_values: numpy array with raw values of Bias current
    :param idb_version: string with idb version
    :param idb_source: string with idb_source
    :return: values in physical units (nA)
    """

    # Retrieve engineering values in uA and return them in nA
    return (
        raw_to_eng(
            raw_values,
            TF_CP_BIA_P011_SRDB_ID,
            idb_source=idb_source,
            idb_version=idb_version,
        )
        * 1000
    )


def sbm_qf_eng(raw_values, tf_srdb_id, idb_source="MIB", idb_version=None):
    """
    Retrieve engineering values of the SBM1/SBM2 event quality factor

    :param raw_values: SBM1 QF raw values
    :param tf_srdb_id: SBM1/SBM2 F Transfer function SRDB ID (i.e, TF_PA_DPU_0038 or =TF_PA_DPU_0039)
    :param idb_source:
    :param idb_version:
    :return: engineering values of SBM1 QF
    """
    return raw_to_eng(
        raw_values, tf_srdb_id, idb_source=idb_source, idb_version=idb_version
    )


def get_current_idb(idb_source, session, tryouts=TRYOUTS, wait=TIME_WAIT_SEC):
    """
    Get current idb release stored in the database

    :param idb_source: IDB source to use (MIB, SRDB or PALISADE).
    :param session: database session
    :param tryouts: number of tries
    :param wait: seconds to wait between two tries
    :return: version of the idb tagged as current, None if not found
    """
    idb_version = None

    filters = [IdbRelease.idb_source == idb_source, IdbRelease.current == true()]
    for i in range(tryouts):
        try:
            query = session.query(IdbRelease.idb_version).filter(and_(*filters))
            results = query.one()
        except MultipleResultsFound:
            logger.error(f"Multiple results found for {actual_sql(query)}!")
            break
        except NoResultFound:
            logger.info(f"No result found for {actual_sql(query)}")
            break
        except Exception as e:
            logger.error(
                f"Cannot run {actual_sql(query)} (trying again in {wait} seconds)"
            )
            logger.debug(e)
            time.sleep(wait)
        else:
            idb_version = results.idb_version
            break

    return idb_version


def gen_sql_filters(model, start_time=None, end_time=None, field="utc_time"):
    """
    Generate common filters for query.

    :param model: table class
    :start_time: query rows greater or equal than start_time only (datetime object)
    :end_time: query rows lesser than end_time only (datetime object)
    :param field: field to use for filters
    :return: list of filters
    """
    filters = []
    if start_time:
        filters.append(model.__dict__[field] >= start_time)
    if end_time:
        filters.append(model.__dict__[field] < end_time)

    return and_(*filters)


def compute_hfr_list_freq(freq_index):
    """
    In HFR LIST mode, return frequency value in kHz giving its
    index

    :param freq_index: index of the frequency
    :return: Value of the frequency in kHz
    """
    return 375 + 50 * (int(freq_index) - 436)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Reference documentation: ROC-GEN-SYS-NTT-00038-LES

"""
Database model for tm_log and tc_log tables.
"""

from poppy.core.db.non_null_column import NonNullColumn

from poppy.core.db.base import Base
from sqlalchemy import String, UniqueConstraint
from sqlalchemy.dialects.postgresql import (
    BIGINT,
    BOOLEAN,
    INTEGER,
    SMALLINT,
    TIMESTAMP,
    JSONB,
    TEXT,
)

__all__ = [
    "TmLog",
    "TcLog",
    "InvalidPacketLog",
]


class TmLog(Base):
    """
    Class representation of the table for tm_log table in the ROC database.
    """

    id = NonNullColumn(BIGINT(), primary_key=True)
    length = NonNullColumn(INTEGER(), nullable=True, descr="Packet length in bytes")
    category = NonNullColumn(
        String(512), nullable=True, descr="Packet PALISADE category"
    )
    apid = NonNullColumn(INTEGER(), nullable=True, descr="Packet APID")
    sync_flag = NonNullColumn(
        BOOLEAN, nullable=True, descr="TM packet time synchronization flag"
    )
    srdb_id = NonNullColumn(String(16), nullable=True, descr="Packet name (SRDB ID)")
    palisade_id = NonNullColumn(String(256), nullable=True, descr="Packet PALISADE ID")
    binary = NonNullColumn(
        TEXT(), nullable=True, descr="Packet raw binary data (in hexadecimal)"
    )
    data = NonNullColumn(
        JSONB(), nullable=True, descr="Packet source data (JSON format)"
    )
    sha = NonNullColumn(TEXT(), descr="Packet sha (hexdigest)")
    sequence_cnt = NonNullColumn(
        BIGINT(), nullable=True, descr="Packet sequence counter"
    )
    cuc_coarse_time = NonNullColumn(
        BIGINT(), descr="Packet creation time (CUC coarse part)"
    )
    cuc_fine_time = NonNullColumn(
        INTEGER(), nullable=True, descr="Packet creation time (CUC fine part)"
    )
    acq_coarse_time = NonNullColumn(
        BIGINT(), nullable=True, descr="Acquisition time (CUC coarse part)"
    )
    acq_fine_time = NonNullColumn(
        INTEGER(), nullable=True, descr="Acquisition time (CUC fine part)"
    )
    obt_time = NonNullColumn(
        TIMESTAMP,
        nullable=True,
        descr='Packet creation on-board time in SQL timestamp format".',
    )
    utc_time = NonNullColumn(
        TIMESTAMP,
        nullable=True,
        descr='Packet creation UTC time in SQL timestamp format".',
    )
    utc_time_is_predictive = NonNullColumn(
        BOOLEAN,
        nullable=True,
        descr="Flag to indicate if "
        "Packet creation UTC time is predictive (True) "
        "or definitive (False)",
    )
    insert_time = NonNullColumn(TIMESTAMP, descr="Database insertion local time.")

    __tablename__ = "tm_log"
    __table_args__ = (
        UniqueConstraint("sha", "cuc_coarse_time"),
        {
            "schema": "pipeline",
        },
    )


class TcLog(Base):
    """
    Class representation of the table for tc_log table in the ROC database.
    """

    id = NonNullColumn(BIGINT(), primary_key=True)
    length = NonNullColumn(INTEGER(), descr="Packet length in bytes")
    category = NonNullColumn(
        String(512), nullable=True, descr="Packet PALISADE category"
    )
    apid = NonNullColumn(INTEGER(), nullable=True, descr="Packet APID")
    utc_time = NonNullColumn(TIMESTAMP, descr="Packet execution time in UTC")
    srdb_id = NonNullColumn(String(16), descr="Packet name (SRDB ID)")
    palisade_id = NonNullColumn(String(256), descr="Packet PALISADE ID")
    binary = NonNullColumn(
        String(), nullable=True, descr="Packet raw binary data (in hexadecimal)"
    )
    data = NonNullColumn(
        JSONB(), nullable=True, descr="Packet application data (JSON format)"
    )
    sha = NonNullColumn(String(), descr="Packet sha (hexdigest)")
    tc_exe_state = NonNullColumn(
        String(16),
        nullable=True,
        descr="TC acknowledgmentexecution completion status",
    )
    tc_acc_state = NonNullColumn(
        String(16), nullable=True, descr="TC acknowledgment acceptance status"
    )
    sequence_name = NonNullColumn(String(16), nullable=True, descr="TC sequence name")
    unique_id = NonNullColumn(String(256), nullable=True, descr="TC unique ID")
    insert_time = NonNullColumn(TIMESTAMP, descr="Database insertion local time.")

    __tablename__ = "tc_log"
    __table_args__ = (
        UniqueConstraint("sha"),
        {
            "schema": "pipeline",
        },
    )


class InvalidPacketLog(Base):
    """
    Class representation of the table for invalid_packet_log table in the ROC database.
    """

    id = NonNullColumn(BIGINT(), primary_key=True)
    sha = NonNullColumn(
        String(), descr="sha256 computed from invalid packet data (hexdigest)"
    )
    srdb_id = NonNullColumn(String(16), nullable=True, descr="Packet name (SRDB ID)")
    apid = NonNullColumn(INTEGER(), nullable=True, descr="Packet APID")
    palisade_id = NonNullColumn(String(256), nullable=True, descr="Packet PALISADE ID")
    utc_time = NonNullColumn(TIMESTAMP, nullable=True, descr="Packet time in UTC")
    data = NonNullColumn(
        JSONB(),
        nullable=True,
        descr="Packet content data (when extraction is possible)",
    )
    status = NonNullColumn(
        SMALLINT(),
        descr="Status of the packet (INVALID_PACKET_HEADER = 1, "
        "INVALID_PACKET_DATA = 2, OUTSIDE_RANGE_PACKET = 3, "
        "INVALID_PACKET_CDATA = 4, INVALID_PACKET_TIME = 5, "
        "UNKNOWN_STATUS = -1)",
    )
    comment = NonNullColumn(
        String(), nullable=True, descr="Additional comment about packet status"
    )
    insert_time = NonNullColumn(TIMESTAMP, descr="Database insertion local time.")

    __tablename__ = "invalid_packet_log"
    __table_args__ = (
        UniqueConstraint("sha"),
        {
            "schema": "pipeline",
        },
    )



"""empty message

Revision ID: roc_dingo_0021_add_retrieved_time
Revises: roc_dingo_0020_rm_biacurrtablehfr
Create Date: 2022-01-17 15:26:10.806747

"""

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

from poppy.pop.alembic.helpers import execute, user

# revision identifiers, used by Alembic.
revision = "roc_dingo_0021_add_retrieved_time"
down_revision = "roc_dingo_0020_rm_biacurrtablehfr"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "sbm_log",
        sa.Column("retrieved_time", postgresql.TIMESTAMP(), nullable=True),
        schema="pipeline",
    )

    # added by hand
    # fix table permissions
    grant_user_seq = """GRANT USAGE, SELECT
        ON ALL SEQUENCES IN SCHEMA pipeline TO {0}""".format(user)
    execute(grant_user_seq)

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("sbm_log", "retrieved_time", schema="pipeline")
    # ### end Alembic commands ###



# -*- coding: utf-8 -*-
"""Rename column type to level

Revision ID: 0005
Revises: roc_dingo_0004_modify_file_log
Create Date: 2020-10-14 14:37:49.295678

"""

from alembic import op

# revision identifiers, used by Alembic.
revision = "roc_dingo_0005_mv_type_level"
down_revision = "roc_dingo_0004_modify_file_log"
depends_on = None


def upgrade():
    op.execute("DROP TYPE IF EXISTS file_level_list")
    op.execute("ALTER TYPE file_type_list  RENAME TO file_level_list")

    # rename type -> level
    op.alter_column("file_log", "type", new_column_name="level", schema="pipeline")


def downgrade():
    op.execute("ALTER TYPE file_level_list RENAME TO file_type_list")

    # rename level -> type
    op.alter_column("file_log", "level", new_column_name="type", schema="pipeline")
